import get from 'lodash.get';
import { buildYup } from 'json-schema-to-yup';

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];
      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }
    return target;
  };
  return _extends.apply(this, arguments);
}

// Desc: Configs for the translation that is used in the validation messages
const translatesConfig = {
  required: 'validation.required',
  min: 'validation.min',
  max: 'validation.max',
  maxLength: 'validation.max_length',
  minLength: 'validation.min_length',
  pattern: 'validation.pattern',
  format: 'validation.format',
  email: 'validation.email'
};
// Desc: Enum for the validation error types
var EValidationTypes;
(function (EValidationTypes) {
  EValidationTypes["required"] = "required";
  EValidationTypes["min"] = "min";
  EValidationTypes["max"] = "max";
  EValidationTypes["maxLength"] = "maxLength";
  EValidationTypes["minLength"] = "minLength";
  EValidationTypes["pattern"] = "pattern";
  EValidationTypes["format"] = "format";
})(EValidationTypes || (EValidationTypes = {}));

// @ts-ignore
/**
 * @param formDataChildren - array of form fields
 * @param acc - accumulator, used for recursion
 * @returns {[key: string]: any} - object with the initial values for the form
 * @description getInitialValues function returns an object with the initial values for the form
 * based on the initial values provided in the json schema
 */
const getInitialValues = (formDataChildren, acc = {}) => {
  return formDataChildren.reduce((acc, item) => {
    if (isFormGroup(item)) {
      return getInitialValues(item.children, acc);
    }
    acc[item.fieldName] = item.initialValue === null ? undefined : item.initialValue;
    return acc;
  }, acc);
};
/**
 * @param schema - yup schema
 * @param fieldName - field name to get the rule value from
 * @param ruleName - rule name to get the value from
 * @returns {any} - rule value
 * @description getRuleValueFromSchema function returns the value of a rule from a yup schema
 * based on the field name and rule name. This value is needed for passing meta to translations.
 * @example getRuleValueFromSchema(schema, 'email', 'minLength') => 6
 */
const getRuleValueFromSchema = (schema, fieldName, ruleName) => {
  const describedSchema = schema.describe();
  const rules = get(describedSchema, `fields.${fieldName}.tests`, []);
  const rule = rules.find(rule => rule.name === ruleName);
  return get(rule, `params.${ruleName}`, undefined);
};
/**
 * @param type - validation type
 * @param value - value to be validated
 * @param t - translation function(optional)
 * @param validationParam - validation param(meta) to be passed to the translation function(optional) or error object
 * @returns {any} - error object or function, or translated string
 * @description getFieldError function returns an error object or function, or translated string based on the validation type
 */
const getFieldError = (type, value, t, validationParam) => {
  let translationKey = get(translatesConfig, `${type}`);
  switch (type) {
    case EValidationTypes.required:
      return t ? t(translationKey) : createErrorObject(translationKey);
    case EValidationTypes.min:
      return t ? ({
        min
      }) => t(translationKey, {
        param: min
      }) : createErrorObject(translationKey, validationParam);
    case EValidationTypes.max:
      return t ? ({
        max
      }) => t(translationKey, {
        param: max
      }) : createErrorObject(translationKey, validationParam);
    case EValidationTypes.maxLength:
      return t ? ({
        maxLength
      }) => t(translationKey, {
        param: maxLength
      }) : createErrorObject(translationKey, validationParam);
    case EValidationTypes.minLength:
      return t ? ({
        minLength
      }) => t(translationKey, {
        param: minLength
      }) : createErrorObject(translationKey, validationParam);
    case EValidationTypes.pattern:
      return t ? t(translationKey) : createErrorObject(translationKey, validationParam);
    case EValidationTypes.format:
      if (value) {
        translationKey = get(translatesConfig, `${type}`) ? get(translatesConfig, `${type}`) : translationKey;
      }
      return t ? t(translationKey) : createErrorObject(translationKey, validationParam);
    default:
      return null;
  }
};
/**
 * @param form - form json schema
 * @param formValues - form values to be validated
 * @returns {IValidationErrors} - object with the validation errors
 * @description makeCrossFieldValidation function makes custom cross-field validation
 * and returns an object with the validation errors
 */
const makeCrossFieldValidation = (form, formValues) => {
  const errors = {};
  const validationObject = form.validationObject;
  try {
    Object.keys(validationObject.crossFieldValidations).forEach(fieldName => {
      const rules = validationObject.crossFieldValidations[fieldName];
      const fieldErrors = {
        property: fieldName,
        constraints: {}
      };
      rules.forEach((rule, id) => {
        const {
          errorKey,
          testCondition,
          meta
        } = rule;
        if (functionFromString(testCondition)(formValues)) {
          fieldErrors.constraints[id] = createErrorObject(errorKey, meta);
        }
      });
      if (Object.keys(fieldErrors.constraints).length > 0) {
        errors[fieldName] = fieldErrors;
      }
    });
    return errors;
  } catch (e) {
    console.warn('Error in makeCrossFieldValidation: ', e);
    return {};
  }
};
/**
 * @param jsonSchema - json validation schema
 * @param t - translation function(optional)
 * @returns {AnyObjectSchema} - yup validation schema
 * @description buildYupSchema function builds a yup validation schema based on the json validation schema
 */
const buildYupSchema = (jsonSchema, t) => {
  return buildYup(jsonSchema, t ? {
    errMessages: getErrorMessages(jsonSchema, t)
  } : undefined);
};
/**
 * @param form {IForm} - form json schema
 * @param valuesToValidate {any} - form values to be validated
 * @param skipCrossField {boolean} - skip cross-field validation(optional)
 * @returns {Promise<unknown>} - object with the validation errors or error that has occurred during the validation
 * @description validateValues function validates the form values based on the json form object
 */
const validateValues = async (form, valuesToValidate, skipCrossField = false) => {
  let errors = {};
  try {
    const validationSchema = buildYupSchema(form.validationObject);
    if (!skipCrossField) {
      errors = makeCrossFieldValidation(form, valuesToValidate);
    }
    await validationSchema.validate(valuesToValidate, {
      abortEarly: false
    });
  } catch (e) {
    if (get(e, 'name') === 'ValidationError') {
      errors = formatYupValidationErrors(e, errors);
    } else {
      return e;
    }
  }
  return {
    errors: Object.values(errors)
  };
};

/**
 * @param jsonSchema - jsonSchema validation object
 * @param t - i18n translate function
 * @returns {any}
 * @description getErrorMessages function returns an object with the error messages for each field in the form based on the jsonSchema
 */
const getErrorMessages = (jsonSchema, t) => {
  const errMessages = {};
  Object.keys(jsonSchema.properties).forEach(property => {
    errMessages[property] = {};
    Object.keys(jsonSchema.properties[property]).forEach(key => {
      const value = jsonSchema.properties[property][key];
      errMessages[property][key] = getFieldError(key, value, t);
      if (Object.keys(jsonSchema).includes(key)) {
        errMessages[property].required = getFieldError('required', null, t);
      }
    });
  });
  return errMessages;
};
/**
 * @param translateKey
 * @param param
 * @returns {IValidationError}
 * @description createErrorObject function returns an object with the error message
 * for a field in the form in a proper format
 * @example createErrorObject('required', { param: 'email' }) => { translationKey: 'required', fallbackLanguageTranslation: 'This field is required', meta: { param: 'email' } }
 */
const createErrorObject = (translateKey, param) => ({
  translationKey: translateKey,
  fallbackLanguageTranslation: translateKey,
  meta: param ? {
    param
  } : {}
});
function isFormGroup(toBeDetermined) {
  return !!toBeDetermined.children;
}
/**
 * @param stringifiedFunction
 * @returns {Function}
 * @description functionFromString returns a function from a stringified function
 * @example functionFromString('/Function((values) => {return values.medicalConditions.toString() === 'true'})/')
 * => (values) => values.medicalConditions.toString() === 'true'
 */
const functionFromString = stringifiedFunction => {
  try {
    const substr = stringifiedFunction.substring(10, stringifiedFunction.length - 2);
    return (0, eval)('(' + substr + ')');
  } catch (e) {
    console.warn('Error parsing function in functionFromString: ', e);
    return () => false;
  }
};
/**
 * @param validationError - yup validation error
 * @param initialErrors - initial errors object to be merged with the yup validation errors
 * @returns {IValidationErrors} - returns an object with the errors for each field in the form
 * @description formatYupValidationErrors function returns an object with the errors for each field in the form
 * based on the yup validation error formatting to be used in the GenericForm component
 * @example formatYupValidationErrors(yupValidationError, { email: { property: 'email', constraints: { required: { translationKey: 'required', fallbackLanguageTranslation: 'This field is required', meta: {} } } } })
 */
const formatYupValidationErrors = (validationError, initialErrors = {}) => {
  const errors = _extends({}, initialErrors);
  for (const propertyValidationError of validationError.inner) {
    if (!propertyValidationError.type) {
      continue;
    }
    if (!get(errors, `${propertyValidationError.path}`)) {
      errors[propertyValidationError.path] = {
        property: propertyValidationError.path,
        constraints: {}
      };
    }
    errors[propertyValidationError.path].constraints[propertyValidationError.type] = _extends({}, getFieldError(propertyValidationError.type, get(propertyValidationError, 'path', null), undefined, get(propertyValidationError.params, `${propertyValidationError.type}`)), {
      fallbackLanguageTranslation: propertyValidationError.message
    });
  }
  return errors;
};

export { buildYupSchema, createErrorObject, formatYupValidationErrors, functionFromString, getErrorMessages, getFieldError, getInitialValues, getRuleValueFromSchema, isFormGroup, makeCrossFieldValidation, validateValues };
//# sourceMappingURL=index.modern.mjs.map
